/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.common.lock;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.Optional;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.user.common.errorhandling.LockCreationException;

class LockServiceTest {

	@InjectMocks
	private LockService service;
	@Mock
	private LockRepository repository;

	@DisplayName("Is not Locked")
	@Nested
	class TestIsNotLocked {

		@Test
		void shouldCallRepository() {
			service.isNotLocked();

			verify(repository).count();
		}

		@Test
		void shouldReturnTrue() {
			when(repository.count()).thenReturn(0L);

			var isNotLocked = service.isNotLocked();

			assertThat(isNotLocked).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			when(repository.count()).thenReturn(1L);

			var isNotLocked = service.isNotLocked();

			assertThat(isNotLocked).isFalse();
		}
	}

	@DisplayName("Is lock older than a day")
	@Nested
	class TestIsLockOlderThanADay {

		@Test
		void shouldCallSyncLockRepository() {
			service.isLockOlderThanADay();

			verify(repository).findLockBefore(any(Instant.class));
		}

		@Test
		void shouldReturnTrue() {
			var timestamp = Instant.now().minus(1, ChronoUnit.DAYS);
			var syncLock = LockTestFactory.createBuilder().timestamp(timestamp.toEpochMilli()).build();
			when(repository.findLockBefore(any(Instant.class))).thenReturn(Optional.of(syncLock));

			var isOlderthantADay = service.isLockOlderThanADay();

			assertThat(isOlderthantADay).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			when(repository.findLockBefore(any(Instant.class))).thenReturn(Optional.empty());

			var isOlderthantADay = service.isLockOlderThanADay();

			assertThat(isOlderthantADay).isFalse();
		}
	}

	@DisplayName("Unlock")
	@Nested
	class TestUnlock {
		@Test
		void shouldCallRepositoryDeleteAll() {
			service.unlock();

			verify(repository).deleteAll();
		}
	}

	@DisplayName("Lock")
	@Nested
	class TestLock {

		private final Instant LOCK = Instant.now();

		@Test
		void shouldCallRepositoryPersist() {
			service.lock(LOCK);

			verify(repository).persist(any(Lock.class));
		}
	}

	@DisplayName("Get by Timestamp")
	@Nested
	class TestGetByTimestamp {

		private final Instant timestamp = Instant.now();

		@Test
		void shouldCallRepositoryFind() {
			when(repository.findLock(any(Instant.class))).thenReturn(Optional.of(LockTestFactory.create()));

			service.getByTimestamp(timestamp);

			verify(repository).findLock(timestamp);
		}

		@Test
		void shouldThrowExceptionIfNotExists() {
			when(repository.findLock(any(Instant.class))).thenReturn(Optional.empty());

			assertThatThrownBy(() -> service.getByTimestamp(timestamp)).isInstanceOf(LockCreationException.class);
		}
	}
}
