/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.sync;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.time.Instant;
import java.util.stream.Stream;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.user.User;
import de.ozgcloud.user.UserService;
import de.ozgcloud.user.UserTestFactory;
import de.ozgcloud.user.common.lock.LockService;
import de.ozgcloud.user.keycloak.KeycloakUserRemoteService;

class SyncServiceTest {

	@Spy
	@InjectMocks
	private SyncService service;
	@Mock
	private KeycloakUserRemoteService keycloakService;
	@Mock
	private LockService lockService;

	@Mock
	private UserService userService;

	@DisplayName("Do the sync")
	@Nested
	class TestDoSync {

		private final User user = UserTestFactory.create();

		@Captor
		private ArgumentCaptor<User> userCaptor;

		@DisplayName("should get all users from keycloak")
		@Test
		void shouldCallKeycloakService() {
			service.sync(Instant.now().toEpochMilli());

			verify(keycloakService).getAllUsers();
		}

		@Test
		void shouldCallSaveAndSync() {
			var syncTimestamp = Instant.now().toEpochMilli();
			var updatedUser = UserTestFactory.createBuilder().lastSyncTimestamp(syncTimestamp).build();
			when(keycloakService.getAllUsers()).thenReturn(Stream.of(user, user));
			doReturn(updatedUser).when(service).addLastSyncTimestamp(user, syncTimestamp);

			service.sync(syncTimestamp);

			verify(userService, times(2)).saveAndSync(updatedUser);
		}

		@Test
		void shouldAddLasSyncToUser() {
			when(keycloakService.getAllUsers()).thenReturn(Stream.of(user));
			when(userService.saveAndSync(any())).thenReturn(user);

			var timestamp = Instant.now().toEpochMilli();

			service.sync(timestamp);

			verify(userService).saveAndSync(userCaptor.capture());
			assertThat(userCaptor.getValue().getLastSyncTimestamp()).isEqualTo(timestamp);
		}

		@DisplayName("should mark all unsynced users as deleted")
		@Test
		void shouldMarkUnsyncedUsersAsDeleted() {
			service.sync(Instant.now().toEpochMilli());

			verify(userService).markUnsyncedUsersAsDeleted(anyLong());
		}
	}
}